package main

import (
	"context"
	"log"
	"os"
	"regexp"
	"time"

	"net/http"

	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/mongo"
	"go.mongodb.org/mongo-driver/mongo/options"
)

type ShortenerAttr struct {
	Slug string
	URL string
}

type Elem struct {
	Shortener ShortenerAttr
}

var reValid = regexp.MustCompile(os.Getenv("GOREDIRECT_REGEX"))
var mongoUri string = os.Getenv("MONGO_URI")
var databaseName string = os.Getenv("GOREDIRECT_DATABASE")
var collectionName string = os.Getenv("GOREDIRECT_COLLECTION")
var redirectFound string = os.Getenv("GOREDIRECT_NOT_FOUND")
var redirectNotFound string = os.Getenv("GOREDIRECT_NOT_FOUND")

func wrapper(coll *mongo.Collection) func (w http.ResponseWriter, r *http.Request) {
	return func(w http.ResponseWriter, r *http.Request) {
		slug := r.URL.Path[1:]
		match := reValid.MatchString(slug)
		if (!match) {
			w.WriteHeader(400)
		}

		var elem Elem
		if err := coll.FindOne(context.TODO(), bson.M{"shortener.slug": slug}).Decode(&elem); err != nil {
			log.Fatal(err)
		}
		log.Printf("Redirecting %s to %s\n", slug, elem.Shortener.URL)
		http.Redirect(w, r, elem.Shortener.URL, 302)
	}
}

func main() {
	client, err := mongo.NewClient(options.Client().ApplyURI(mongoUri))
	if err != nil {
		log.Fatal(err)
	}
	ctx, _ := context.WithTimeout(context.Background(), 10*time.Second)
	err = client.Connect(ctx)
	if err != nil {
		log.Fatal(err)
	}
	coll := client.Database(databaseName).Collection(collectionName)
	defer client.Disconnect(ctx)
	http.HandleFunc("/", wrapper(coll))
	log.Printf("Starting HTTP server\n")
	err2 := http.ListenAndServe(":80", nil)
	if err2 != nil {
		log.Fatal("ListenAndServe: ", err2)
	}
}
